@@ -17,13 +17,14 @@ from cliff import lister
 from osc_lib.command import command
 from osc_lib import exceptions
 from osc_lib import utils
+from osc_lib.utils import tags as _tag
 from oslo_serialization import jsonutils
 from oslo_utils import uuidutils
 
 from octaviaclient.osc.v2 import constants as const
 from octaviaclient.osc.v2 import utils as v2_utils
 
-PROVISIONING_STATUS = ['ACTIVE', 'DELETED', 'ERROR', 'PENDING_CREATE',
+PROVISIONING_STATUS = ['ACTIVE', 'ERROR', 'PENDING_CREATE',
                        'PENDING_UPDATE', 'PENDING_DELETE']
 
 OPERATING_STATUS = ['ONLINE', 'DRAINING', 'OFFLINE', 'DEGRADED', 'ERROR',
@@ -88,6 +89,20 @@ class CreateLoadBalancer(command.ShowOne):
             metavar='<vip_qos_policy_id>',
             help="Set QoS policy ID for VIP port. Unset with 'None'.",
         )
+        parser.add_argument(
+            '--additional-vip',
+            metavar='subnet-id=<name-or-uuid>[,ip-address=<ip>]',
+            action='append',
+            help="Expose an additional VIP on the load balancer. This "
+                 "parameter can be provided more than once."
+        )
+        parser.add_argument(
+            '--vip-sg-id',
+            metavar='<vip_sg_id>',
+            action='append',
+            help="Set a Custom Security Group for VIP port. This "
+                 "parameter can be provided more than once."
+        )
 
         parser.add_argument(
             '--project',
@@ -129,9 +144,12 @@ class CreateLoadBalancer(command.ShowOne):
         parser.add_argument(
             '--wait',
             action='store_true',
-            help='Wait for action to complete',
+            help='Wait for action to complete.',
         )
 
+        _tag.add_tag_option_to_parser_for_create(
+            parser, 'load balancer')
+
         return parser
 
     def take_action(self, parsed_args):
@@ -156,10 +174,15 @@ class CreateLoadBalancer(command.ShowOne):
                         data['loadbalancer']['id']))
             }
 
+        # Handle older API versions that did not have the vip_vnic_type
+        if not data['loadbalancer'].get('vip_vnic_type', False):
+            data['loadbalancer']['vip_vnic_type'] = 'normal'
+
         formatters = {
             'listeners': v2_utils.format_list,
             'pools': v2_utils.format_list,
-            'l7policies': v2_utils.format_list
+            'l7policies': v2_utils.format_list,
+            'tags': v2_utils.format_list_flat
         }
 
         return (rows,
@@ -176,7 +199,7 @@ class DeleteLoadBalancer(command.Command):
         parser.add_argument(
             'loadbalancer',
             metavar='<load_balancer>',
-            help="Load balancers to delete (name or ID)"
+            help="Load balancers to delete (name or ID)."
         )
         parser.add_argument(
             '--cascade',
@@ -188,7 +211,7 @@ class DeleteLoadBalancer(command.Command):
         parser.add_argument(
             '--wait',
             action='store_true',
-            help='Wait for action to complete',
+            help='Wait for action to complete.',
         )
 
         return parser
@@ -223,7 +246,7 @@ class FailoverLoadBalancer(command.Command):
         parser.add_argument(
             '--wait',
             action='store_true',
-            help='Wait for action to complete',
+            help='Wait for action to complete.',
         )
 
         return parser
@@ -326,6 +349,8 @@ class ListLoadBalancer(lister.Lister):
             help="List load balancers according to their availability zone."
         )
 
+        _tag.add_tag_filtering_option_to_parser(parser, 'load balancer')
+
         return parser
 
     def take_action(self, parsed_args):
@@ -377,10 +402,15 @@ class ShowLoadBalancer(command.ShowOne):
             data = self.app.client_manager.load_balancer.load_balancer_show(
                 lb_id=lb_id)
 
+        # Handle older API versions that did not have the vip_vnic_type
+        if not data.get('vip_vnic_type', False):
+            data['vip_vnic_type'] = 'normal'
+
         formatters = {
             'listeners': v2_utils.format_list,
             'pools': v2_utils.format_list,
-            'l7policies': v2_utils.format_list
+            'l7policies': v2_utils.format_list,
+            'tags': v2_utils.format_list_flat
         }
 
         return (rows, (utils.get_dict_properties(
@@ -413,6 +443,13 @@ class SetLoadBalancer(command.Command):
             metavar='<vip_qos_policy_id>',
             help="Set QoS policy ID for VIP port. Unset with 'None'.",
         )
+        parser.add_argument(
+            '--vip-sg-id',
+            metavar='<vip_sg_id>',
+            action='append',
+            help="Set a Custom Security Group for VIP port. This "
+                 "parameter can be provided more than once."
+        )
 
         admin_group = parser.add_mutually_exclusive_group()
         admin_group.add_argument(
@@ -430,15 +467,22 @@ class SetLoadBalancer(command.Command):
         parser.add_argument(
             '--wait',
             action='store_true',
-            help='Wait for action to complete',
+            help='Wait for action to complete.',
         )
 
+        _tag.add_tag_option_to_parser_for_set(parser, 'load balancer')
+
         return parser
 
     def take_action(self, parsed_args):
         attrs = v2_utils.get_loadbalancer_attrs(self.app.client_manager,
                                                 parsed_args)
         lb_id = attrs.pop('loadbalancer_id')
+
+        v2_utils.set_tags_for_set(
+            self.app.client_manager.load_balancer.load_balancer_show,
+            lb_id, attrs, clear_tags=parsed_args.no_tag)
+
         body = {'loadbalancer': attrs}
 
         self.app.client_manager.load_balancer.load_balancer_set(
@@ -478,23 +522,35 @@ class UnsetLoadBalancer(command.Command):
             action='store_true',
             help="Clear the load balancer QoS policy.",
         )
+        parser.add_argument(
+            '--vip-sg-id',
+            dest='vip_sg_ids',
+            action='store_true',
+            help="Clear the Custom Security Groups.",
+        )
         parser.add_argument(
             '--wait',
             action='store_true',
-            help='Wait for action to complete',
+            help='Wait for action to complete.',
         )
 
+        _tag.add_tag_option_to_parser_for_unset(parser, 'load balancer')
+
         return parser
 
     def take_action(self, parsed_args):
         unset_args = v2_utils.get_unsets(parsed_args)
-        if not unset_args:
+        if not unset_args and not parsed_args.all_tag:
             return
 
         lb_id = v2_utils.get_resource_id(
             self.app.client_manager.load_balancer.load_balancer_list,
             'loadbalancers', parsed_args.loadbalancer)
 
+        v2_utils.set_tags_for_unset(
+            self.app.client_manager.load_balancer.load_balancer_show,
+            lb_id, unset_args, clear_tags=parsed_args.all_tag)
+
         body = {'loadbalancer': unset_args}
 
         self.app.client_manager.load_balancer.load_balancer_set(
